import { Test, TestingModule } from '@nestjs/testing';
import { GroupsService } from './groups.service';
import { NotFoundException } from '@nestjs/common';
import { DRIZZLE } from '../../../database/database.module';

describe('GroupsService', () => {
  let service: GroupsService;
  let mockDb: any;

  // Mock data
  const mockGroup = {
    id: 'group1',
    name: 'Test Group',
    projectId: 'project1',
    metadata: {},
    createdAt: new Date(),
    updatedAt: new Date(),
  };

  const mockPerson = {
    id: 'person1',
    name: 'Test Person',
    projectId: 'project1',
    createdAt: new Date(),
    updatedAt: new Date(),
  };

  const mockPersonToGroup = {
    personId: 'person1',
    groupId: 'group1',
  };

  // Mock database operations
  const mockDbOperations = {
    select: jest.fn().mockReturnThis(),
    from: jest.fn().mockReturnThis(),
    where: jest.fn().mockReturnThis(),
    insert: jest.fn().mockReturnThis(),
    values: jest.fn().mockReturnThis(),
    update: jest.fn().mockReturnThis(),
    set: jest.fn().mockReturnThis(),
    delete: jest.fn().mockReturnThis(),
    innerJoin: jest.fn().mockReturnThis(),
    returning: jest.fn().mockImplementation(() => {
      return [mockGroup];
    }),
  };

  beforeEach(async () => {
    mockDb = {
      ...mockDbOperations,
    };

    const module: TestingModule = await Test.createTestingModule({
      providers: [
        GroupsService,
        {
          provide: DRIZZLE,
          useValue: mockDb,
        },
      ],
    }).compile();

    service = module.get<GroupsService>(GroupsService);
  });

  afterEach(() => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  describe('create', () => {
    it('should create a new group', async () => {
      const createGroupDto = {
        name: 'Test Group',
        projectId: 'project1',
        metadata: {},
      };

      const result = await service.create(createGroupDto);

      expect(mockDb.insert).toHaveBeenCalled();
      expect(mockDb.values).toHaveBeenCalledWith({
        ...createGroupDto,
      });
      expect(result).toEqual(mockGroup);
    });
  });

  describe('findAll', () => {
    it('should return all groups', async () => {
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => [mockGroup]);

      const result = await service.findAll();

      expect(mockDb.select).toHaveBeenCalled();
      expect(mockDb.from).toHaveBeenCalled();
      expect(result).toEqual([mockGroup]);
    });
  });

  describe('findByProjectId', () => {
    it('should return groups for a specific project', async () => {
      const projectId = 'project1';
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [mockGroup]);

      const result = await service.findByProjectId(projectId);

      expect(mockDb.select).toHaveBeenCalled();
      expect(mockDb.from).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual([mockGroup]);
    });
  });

  describe('findById', () => {
    it('should return a group by id', async () => {
      const id = 'group1';
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [mockGroup]);

      const result = await service.findById(id);

      expect(mockDb.select).toHaveBeenCalled();
      expect(mockDb.from).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual(mockGroup);
    });

    it('should throw NotFoundException if group not found', async () => {
      const id = 'nonexistent';
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [undefined]);

      await expect(service.findById(id)).rejects.toThrow(NotFoundException);
    });
  });

  describe('update', () => {
    it('should update a group', async () => {
      const id = 'group1';
      const updateGroupDto = {
        name: 'Updated Group',
      };

      const result = await service.update(id, updateGroupDto);

      expect(mockDb.update).toHaveBeenCalled();
      expect(mockDb.set).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual(mockGroup);
    });

    it('should throw NotFoundException if group not found', async () => {
      const id = 'nonexistent';
      const updateGroupDto = {
        name: 'Updated Group',
      };

      mockDb.update.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.set.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => [undefined]);

      await expect(service.update(id, updateGroupDto)).rejects.toThrow(NotFoundException);
    });
  });

  describe('remove', () => {
    it('should remove a group', async () => {
      const id = 'group1';

      const result = await service.remove(id);

      expect(mockDb.delete).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual(mockGroup);
    });

    it('should throw NotFoundException if group not found', async () => {
      const id = 'nonexistent';

      mockDb.delete.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => [undefined]);

      await expect(service.remove(id)).rejects.toThrow(NotFoundException);
    });
  });

  describe('addPersonToGroup', () => {
    it('should add a person to a group', async () => {
      const groupId = 'group1';
      const personId = 'person1';

      // Mock findById to return the group
      jest.spyOn(service, 'findById').mockResolvedValueOnce(mockGroup);

      // Reset and setup mocks for this test
      jest.clearAllMocks();

      // Mock person lookup
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [[mockPerson]]);

      // Mock relation lookup
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [undefined]);

      // Mock relation creation
      mockDb.insert.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.values.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => [mockPersonToGroup]);

      const result = await service.addPersonToGroup(groupId, personId);

      expect(service.findById).toHaveBeenCalledWith(groupId);
      expect(mockDb.select).toHaveBeenCalled();
      expect(mockDb.from).toHaveBeenCalled();
      expect(mockDb.insert).toHaveBeenCalled();
      expect(mockDb.values).toHaveBeenCalledWith({
        personId,
        groupId,
      });
      expect(result).toEqual(mockPersonToGroup);
    });

    it('should throw NotFoundException if person not found', async () => {
      const groupId = 'group1';
      const personId = 'nonexistent';

      // Mock findById to return the group
      jest.spyOn(service, 'findById').mockResolvedValueOnce(mockGroup);

      // Reset and setup mocks for this test
      jest.clearAllMocks();

      // Mock person lookup to return no person
      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => [undefined]);

      await expect(service.addPersonToGroup(groupId, personId)).rejects.toThrow(NotFoundException);
    });
  });

  describe('removePersonFromGroup', () => {
    it('should remove a person from a group', async () => {
      const groupId = 'group1';
      const personId = 'person1';

      // Reset and setup mocks for this test
      jest.clearAllMocks();

      mockDb.delete.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => [mockPersonToGroup]);

      const result = await service.removePersonFromGroup(groupId, personId);

      expect(mockDb.delete).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual(mockPersonToGroup);
    });

    it('should throw NotFoundException if relation not found', async () => {
      const groupId = 'group1';
      const personId = 'nonexistent';

      // Reset and setup mocks for this test
      jest.clearAllMocks();

      mockDb.delete.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.returning.mockImplementationOnce(() => [undefined]);

      await expect(service.removePersonFromGroup(groupId, personId)).rejects.toThrow(NotFoundException);
    });
  });

  describe('getPersonsInGroup', () => {
    it('should get all persons in a group', async () => {
      const groupId = 'group1';
      const mockPersons = [{ person: mockPerson }];

      // Mock findById to return the group
      jest.spyOn(service, 'findById').mockResolvedValueOnce(mockGroup);

      mockDb.select.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.from.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.innerJoin.mockImplementationOnce(() => mockDbOperations);
      mockDbOperations.where.mockImplementationOnce(() => mockPersons);

      const result = await service.getPersonsInGroup(groupId);

      expect(service.findById).toHaveBeenCalledWith(groupId);
      expect(mockDb.select).toHaveBeenCalled();
      expect(mockDb.from).toHaveBeenCalled();
      expect(mockDb.innerJoin).toHaveBeenCalled();
      expect(mockDb.where).toHaveBeenCalled();
      expect(result).toEqual(mockPersons);
    });
  });
});
